unit uDollsFactory;

interface

uses
  uToyFactory
  , uToy
  , classes                         // for RegisterClass & TList
  ;

type

  TDollsFactory = class(TToyFactory)
  private

    FFreeList: TList;
    FNextToy: integer;

  protected

    function Borrow: TToy; override;

  public

    procedure Return(Toy: TToy); override;

    constructor Create; override;
    destructor Destroy; override;

    function GetSet1: TToy; override;
    function GetSet2: TToy; override;
    function GetSet3: TToy; override;

  published

  end;

implementation

uses
  uDolls
  , sysutils                            // for Exception
  ;

{ TDollsFactory }

const
  nMaxToys: integer = 7;

function TDollsFactory.Borrow: TToy;
begin
  if FNextToy > -1 then
  begin
    Result := TToy(FFreeList.Items[FNextToy]);
    FFreeList.Items[FNextToy] := nil;           // not essential to do this
    Dec(FNextToy);
  end
  else
  begin
    raise Exception.Create('There are no more dolls in the library');
  end;
end;

constructor TDollsFactory.Create;
var
  nToy: integer;
begin
  FFreeList := TList.Create;
  for nToy := 0 to Pred(nMaxToys) do
    FFreeList.Add(TDolls.Create);
  FNextToy := Pred(FFreeList.Count);
end;

destructor TDollsFactory.Destroy;
var
  nToy: integer;
begin
  // Assume that no toys are still out on loan
  for nToy := 0 to FNextToy do
    TDolls(FFreeList.Items[nToy]).Free;
  FFreeList.Free;
  inherited;
end;

function TDollsFactory.GetSet1: TToy;
begin
  Result := Self.Borrow;
  Result.SetNo := 1;
end;

function TDollsFactory.GetSet2: TToy;
begin
  Result := Self.Borrow;
  Result.SetNo := 2;
end;

function TDollsFactory.GetSet3: TToy;
begin
  Result := Self.Borrow;
  Result.SetNo := 3;
end;

procedure TDollsFactory.Return(Toy: TToy);
begin
  if (FNextToy < Pred(FFreeList.Count)) then
  begin
    Inc(FNextToy);
    FFreeList.Items[FNextToy] := Toy;
  end
  else
  begin
    // This should never happen
    raise Exception.Create('No room in the library to return this toy');
  end;
end;

initialization
  RegisterClass(TDollsFactory);
  RegisterFactory('Dolls', 'TDollsFactory');

end.
